/* 
Querying Extended Events Target XML

Jes Borland, Brent Ozar Unlimited
v1.0 - January 2015

(C) 2014, Brent Ozar Unlimited. 
See http://BrentOzar.com/go/eula for the End User Licensing Agreement.
*/

/* This demo uses the ring buffer - this can impact performance! Use with caution. */ 

CREATE EVENT SESSION [Ring Buffer - Track Waits] ON SERVER 
ADD EVENT sqlos.wait_info(
    ACTION(sqlserver.database_id,sqlserver.database_name,sqlserver.is_system,sqlserver.session_id,sqlserver.sql_text)) 
ADD TARGET package0.ring_buffer
WITH (STARTUP_STATE=OFF)
GO

/* Start session */
ALTER EVENT SESSION [Ring Buffer - Track Waits] ON SERVER STATE=START;

/* Run a workload or series of queries. */ 
USE AdventureWorks2012;
GO 
SELECT TOP 10 p.ProductID, 
	SUM(th.Quantity)
FROM dbo.bigProduct p 
	INNER JOIN dbo.bigTransactionHistory th ON p.ProductID = th.ProductID 
GROUP BY p.ProductID; 

SELECT COUNT(soh.SalesOrderID), 
	MAX(soh.OrderDate), 
	soh.CustomerID 
FROM Sales.SalesOrderHeader soh 
	INNER JOIN Sales.SalesOrderDetail sod ON soh.SalesOrderID = sod.SalesOrderID
WHERE sod.ProductID = 36877 
GROUP BY soh.CustomerID;

/* Query to view output - from BOL 
http://msdn.microsoft.com/en-us/library/ff878182.aspx */ 
SELECT name, target_name, CAST(xet.target_data AS xml)
FROM sys.dm_xe_session_targets AS xet
JOIN sys.dm_xe_sessions AS xe
   ON (xe.address = xet.event_session_address)
WHERE xe.name = 'Ring Buffer - Track Waits'; 
/* Wow, is THAT ever ugly */

/* Break this down, step by step */

/*Remember, the output of the ring buffer is XML. The first thing we need to do is get it into a format we can use. 
Options include temp tables, CTEs, and even XML variables. 
Let's use a temp table. */
SELECT CAST(target_data as xml) AS targetdata
INTO #capture_waits_data
FROM sys.dm_xe_session_targets xet
	JOIN sys.dm_xe_sessions xes ON xes.address = xet.event_session_address
WHERE xes.name = 'Ring Buffer - Track Waits'
	AND xet.target_name = 'ring_buffer';

/*What does that show us? */
SELECT * 
FROM #capture_waits_data;
/* Open the link. 
How ugly is that? */

/* Now we start "shredding" XML. 
Need some help with this? So does...everyone. 
BOL: http://msdn.microsoft.com/en-us/library/ms189075.aspx 
Look at examples and hack your way through it. */

/* Here, we start by querying the data name="wait_type" node for the "text" value - the name. */ 
SELECT xed.event_data.value('(data[@name="wait_type"]/text)[1]', 'varchar(25)') AS wait_type
FROM #capture_waits_data
	CROSS APPLY targetdata.nodes('//RingBufferTarget/event') AS xed (event_data); 
/*So now we just want to apply this to the other "values" we want to parse out. */

/*Date, wait type, duration, signal duration. */
SELECT xed.event_data.value('(@timestamp)[1]', 'datetime2') AS [timestamp],
	xed.event_data.value('(data[@name="wait_type"]/text)[1]', 'varchar(25)') AS wait_type, 
	xed.event_data.value('(data[@name="duration"]/value)[1]', 'int') AS wait_type_duration_ms, 
	xed.event_data.value('(data[@name="signal_duration"]/value)[1]', 'int') AS wait_type_signal_duration_ms 
FROM #capture_waits_data
	CROSS APPLY targetdata.nodes('//RingBufferTarget/event') AS xed (event_data); 
/*Cool! */

/*But we want to view totals. Can we aggregate? */
SELECT xed.event_data.value('(data[@name="wait_type"]/text)[1]', 'varchar(25)') AS wait_type, 
	COUNT (xed.event_data.value('(data[@name="wait_type"]/text)[1]', 'varchar(25)')) AS count_wait_type, 
	SUM (xed.event_data.value('(data[@name="duration"]/value)[1]', 'int')) AS sum_wait_type_duration_ms, 
	SUM (xed.event_data.value('(data[@name="signal_duration"]/value)[1]', 'int')) AS sum_wait_type_signal_duration_ms 
FROM #capture_waits_data
	CROSS APPLY targetdata.nodes('//RingBufferTarget/event') AS xed (event_data) 
GROUP BY xed.event_data.value('(data[@name="wait_type"]/text)[1]', 'varchar(25)');

/*OK, let's try another approach - a derived table. 
Get the unaggregated data first, so we can see how that looks. */
SELECT xei.wait_type, 
	xei.wait_type_duration_ms, 
	xei.wait_type_signal_duration_ms 
FROM 
	(SELECT xed.event_data.value('(@timestamp)[1]', 'datetime2') AS [timestamp],
		xed.event_data.value('(data[@name="wait_type"]/text)[1]', 'varchar(25)') AS wait_type, 
		xed.event_data.value('(data[@name="duration"]/value)[1]', 'int') AS wait_type_duration_ms, 
		xed.event_data.value('(data[@name="signal_duration"]/value)[1]', 'int') AS wait_type_signal_duration_ms 
	FROM #capture_waits_data
		CROSS APPLY targetdata.nodes('//RingBufferTarget/event') AS xed (event_data)) AS xei; 
/*With aggregations. */
SELECT xei.wait_type, 
	COUNT(xei.wait_type) AS count_wait_type, 
	SUM(xei.wait_type_duration_ms) AS sum_wait_type_duration_ms, 
	SUM(xei.wait_type_signal_duration_ms) AS sum_wait_type_signal_duration_ms 
FROM 
	(SELECT xed.event_data.value('(@timestamp)[1]', 'datetime2') AS [timestamp],
		xed.event_data.value('(data[@name="wait_type"]/text)[1]', 'varchar(25)') AS wait_type, 
		xed.event_data.value('(data[@name="duration"]/value)[1]', 'int') AS wait_type_duration_ms, 
		xed.event_data.value('(data[@name="signal_duration"]/value)[1]', 'int') AS wait_type_signal_duration_ms 
	FROM #capture_waits_data
		CROSS APPLY targetdata.nodes('//RingBufferTarget/event') AS xed (event_data)) AS xei 
GROUP BY xei.wait_type
ORDER BY SUM(xei.wait_type_duration_ms) DESC; 

/* Clean up */ 
DROP TABLE #capture_waits_data; 
GO 
/* Stop session */
ALTER EVENT SESSION [Ring Buffer - Track Waits] ON SERVER STATE=STOP;
/* Drop session */
DROP EVENT SESSION [Ring Buffer - Track Waits] ON SERVER; 